@@ -45,17 +45,25 @@
 
 #include <gdk/gdk.h>
 
-#if defined(HAVE_CLUTTER_GTK_X11)
-
+#if defined(CLUTTER_WINDOWING_X11)
 #include <clutter/x11/clutter-x11.h>
-#include <gdk/gdkx.h>
+#endif
 
-#elif defined(HAVE_CLUTTER_GTK_WIN32)
+#if defined(CLUTTER_WINDOWING_GDK)
+#include <clutter/gdk/clutter-gdk.h>
+#endif
 
+#if defined(CLUTTER_WINDOWING_WIN32)
 #include <clutter/win32/clutter-win32.h>
-#include <gdk/gdkwin32.h>
+#endif
 
-#endif /* HAVE_CLUTTER_GTK_{X11,WIN32} */
+#if defined(GDK_WINDOWING_X11)
+#include <gdk/gdkx.h>
+#endif
+
+#if defined(GDK_WINDOWING_WIN32)
+#include <gdk/gdkwin32.h>
+#endif
 
 G_DEFINE_TYPE (GtkClutterEmbed, gtk_clutter_embed, GTK_TYPE_CONTAINER);
 
@@ -183,12 +191,26 @@ gtk_clutter_filter_func (GdkXEvent *native_event,
                          GdkEvent  *event         G_GNUC_UNUSED,
                          gpointer   user_data     G_GNUC_UNUSED)
 {
-#ifdef HAVE_CLUTTER_GTK_X11
-  XEvent *xevent = native_event;
+#if defined(CLUTTER_WINDOWING_X11)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11))
+    {
+      XEvent *xevent = native_event;
 
-  /* let Clutter handle all events coming from the windowing system */
-  clutter_x11_handle_event (xevent);
+      /* let Clutter handle all events coming from the windowing system */
+      clutter_x11_handle_event (xevent);
+    }
+  else
 #endif
+#if defined(CLUTTER_WINDOWING_WIN32)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WIN32))
+    {
+      MSG *msg = native_event;
+
+      clutter_win32_handle_event (msg);
+    }
+  else
+#endif
+    g_critical ("Unsuppored Clutter backend");
 
   /* we don't care if Clutter handled the event: we want GDK to continue
    * the event processing as usual
@@ -207,26 +229,27 @@ gtk_clutter_embed_realize (GtkWidget *widget)
   gint attributes_mask;
   gint border_width;
 
-#ifdef HAVE_CLUTTER_GTK_X11
-  {
-    const XVisualInfo *xvinfo;
-    GdkVisual *visual;
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11))
+    {
+      const XVisualInfo *xvinfo;
+      GdkVisual *visual;
 
-    /* We need to use the colormap from the Clutter visual, since
-     * the visual is tied to the GLX context
-     */
-    xvinfo = clutter_x11_get_visual_info ();
-    if (xvinfo == None)
-      {
-        g_critical ("Unable to retrieve the XVisualInfo from Clutter");
-        return;
-      }
+      /* We need to use the colormap from the Clutter visual, since
+       * the visual is tied to the GLX context
+       */
+      xvinfo = clutter_x11_get_visual_info ();
+      if (xvinfo == None)
+        {
+          g_critical ("Unable to retrieve the XVisualInfo from Clutter");
+          return;
+        }
 
-    visual = gdk_x11_screen_lookup_visual (gtk_widget_get_screen (widget),
-                                           xvinfo->visualid);
-    gtk_widget_set_visual (widget, visual);
-  }
-#endif /* HAVE_CLUTTER_GTK_X11 */
+      visual = gdk_x11_screen_lookup_visual (gtk_widget_get_screen (widget),
+                                             xvinfo->visualid);
+      gtk_widget_set_visual (widget, visual);
+    }
+#endif
 
   gtk_widget_set_realized (widget, TRUE);
 
@@ -259,6 +282,7 @@ gtk_clutter_embed_realize (GtkWidget *widget)
   window = gdk_window_new (gtk_widget_get_parent_window (widget),
                            &attributes,
                            attributes_mask);
+
   gtk_widget_set_window (widget, window);
   gdk_window_set_user_data (window, widget);
 
@@ -272,15 +296,44 @@ gtk_clutter_embed_realize (GtkWidget *widget)
   style_context = gtk_widget_get_style_context (widget);
   gtk_style_context_set_background (style_context, window);
 
-  gdk_window_add_filter (NULL, gtk_clutter_filter_func, widget);
+#if defined(CLUTTER_WINDOWING_GDK)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_GDK))
+    {
+      clutter_gdk_set_stage_foreign (CLUTTER_STAGE (priv->stage), window);
+    }
+  else
+#endif
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11) &&
+      GDK_IS_X11_WINDOW (window))
+    {
+      static gboolean has_filter = FALSE;
 
-#if defined(HAVE_CLUTTER_GTK_X11)
-  clutter_x11_set_stage_foreign (CLUTTER_STAGE (priv->stage), 
-                                 GDK_WINDOW_XID (window));
-#elif defined(HAVE_CLUTTER_GTK_WIN32)
-  clutter_win32_set_stage_foreign (CLUTTER_STAGE (priv->stage), 
-				   GDK_WINDOW_HWND (window));
-#endif /* HAVE_CLUTTER_GTK_{X11,WIN32} */
+      clutter_x11_set_stage_foreign (CLUTTER_STAGE (priv->stage), GDK_WINDOW_XID (window));
+
+      if (G_UNLIKELY (!has_filter))
+        {
+          gdk_window_add_filter (NULL, gtk_clutter_filter_func, widget);
+          has_filter = TRUE;
+        }
+    }
+  else
+#endif
+#if defined(GDK_WINDOWING_WIN32) && defined(CLUTTER_WINDOWING_WIN32)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WIN32) &&
+      GDK_IS_WIN32_WINDOW (window))
+    {
+      static gboolean has_filter = FALSE;
+
+      clutter_win32_set_stage_foreign (CLUTTER_STAGE (priv->stage), GDK_WINDOW_HWND (window));
+
+      if (G_UNLIKELY (!has_filter))
+        {
+          gdk_window_add_filter (NULL, gtk_clutter_filter_func, widget);
+          has_filter = TRUE;
+        }
+    }
+#endif
 
   clutter_actor_realize (priv->stage);
 
@@ -344,6 +397,8 @@ gtk_clutter_embed_map_event (GtkWidget	 *widget,
 
   clutter_actor_map (priv->stage);
 
+  clutter_actor_queue_redraw (priv->stage);
+
   return res;
 }
 
@@ -444,10 +499,10 @@ gtk_clutter_embed_style_updated (GtkWidget *widget)
   ClutterSettings *clutter_settings;
   gchar *font_name;
   gint double_click_time, double_click_distance;
-#if HAVE_CLUTTER_GTK_X11
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
   gint xft_dpi, xft_hinting, xft_antialias;
   gchar *xft_hintstyle, *xft_rgba;
-#endif /* HAVE_CLUTTER_GTK_X11 */
+#endif
 
   if (gtk_widget_get_realized (widget))
     {
@@ -484,15 +539,21 @@ gtk_clutter_embed_style_updated (GtkWidget *widget)
                 "gtk-font-name", &font_name,
                 "gtk-double-click-time", &double_click_time,
                 "gtk-double-click-distance", &double_click_distance,
-#if HAVE_CLUTTER_GTK_X11
-                "gtk-xft-dpi", &xft_dpi,
-                "gtk-xft-antialias", &xft_antialias,
-                "gtk-xft-hinting", &xft_hinting,
-                "gtk-xft-hintstyle", &xft_hintstyle,
-                "gtk-xft-rgba", &xft_rgba,
-#endif /* HAVE_CLUTTER_GTK_X11 */
                 NULL);
 
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
+  if (GDK_IS_X11_SCREEN (screen))
+    {
+      g_object_get (G_OBJECT (gtk_settings),
+                    "gtk-xft-dpi", &xft_dpi,
+                    "gtk-xft-antialias", &xft_antialias,
+                    "gtk-xft-hinting", &xft_hinting,
+                    "gtk-xft-hintstyle", &xft_hintstyle,
+                    "gtk-xft-rgba", &xft_rgba,
+                    NULL);
+    }
+#endif
+
   /* copy all settings and values coming from GTK+ into
    * the ClutterBackend; this way, a scene embedded into
    * a GtkClutterEmbed will not look completely alien
@@ -502,19 +563,19 @@ gtk_clutter_embed_style_updated (GtkWidget *widget)
                 "font-name", font_name,
                 "double-click-time", double_click_time,
                 "double-click-distance", double_click_distance,
-#if HAVE_CLUTTER_GTK_X11
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11) 
                 "font-antialias", xft_antialias,
                 "font-dpi", xft_dpi,
                 "font-hinting", xft_hinting,
                 "font-hint-style", xft_hintstyle,
                 "font-subpixel-order", xft_rgba,
-#endif /* HAVE_CLUTTER_GTK_X11 */
+#endif
                 NULL);
 
-#if HAVE_CLUTTER_GTK_X11
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
   g_free (xft_hintstyle);
   g_free (xft_rgba);
-#endif /* HAVE_CLUTTER_GTK_X11 */
+#endif
 
   g_free (font_name);
 
@@ -595,6 +656,18 @@ gtk_clutter_embed_child_type (GtkContainer *container)
   return GTK_CLUTTER_TYPE_OFFSCREEN;
 }
 
+static gboolean
+gtk_clutter_embed_event (GtkWidget *widget,
+                         GdkEvent  *event)
+{
+#if defined(CLUTTER_WINDOWING_GDK)
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_GDK))
+    clutter_gdk_handle_event (event);
+#endif
+
+  return FALSE;
+}
+
 static void
 gtk_clutter_embed_class_init (GtkClutterEmbedClass *klass)
 {
@@ -619,6 +692,7 @@ gtk_clutter_embed_class_init (GtkClutterEmbedClass *klass)
   widget_class->focus_out_event = gtk_clutter_embed_focus_out;
   widget_class->key_press_event = gtk_clutter_embed_key_event;
   widget_class->key_release_event = gtk_clutter_embed_key_event;
+  widget_class->event = gtk_clutter_embed_event;
 
   container_class->add = gtk_clutter_embed_add;
   container_class->remove = gtk_clutter_embed_remove;
@@ -630,16 +704,27 @@ static void
 gtk_clutter_embed_init (GtkClutterEmbed *embed)
 {
   GtkClutterEmbedPrivate *priv;
+  GtkWidget *widget;
 
   embed->priv = priv = GTK_CLUTTER_EMBED_GET_PRIVATE (embed);
 
-  gtk_widget_set_can_focus (GTK_WIDGET (embed), TRUE);
-  gtk_widget_set_has_window (GTK_WIDGET (embed), TRUE);
+  widget = GTK_WIDGET (embed);
 
-  /* disable double-buffering: it's automatically provided
-   * by OpenGL
-   */
-  gtk_widget_set_double_buffered (GTK_WIDGET (embed), FALSE);
+  /* we have a real window backing our drawing */
+  gtk_widget_set_has_window (widget, TRUE);
+
+  /* we accept key focus */
+  gtk_widget_set_can_focus (widget, TRUE);
+
+  /* disable double-buffering: it's automatically provided by OpenGL */
+  gtk_widget_set_double_buffered (widget, FALSE);
+
+  /* we own the whole drawing of this widget, including the background */
+  gtk_widget_set_app_paintable (widget, TRUE);
+
+  /* this widget should expand in both directions */
+  gtk_widget_set_hexpand (widget, TRUE);
+  gtk_widget_set_vexpand (widget, TRUE);
 
   /* we always create new stages rather than use the default */
   priv->stage = clutter_stage_new ();
